/*
 * Copyright 2016 The BigDL Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intel.analytics.bigdl.nn.ops

import com.intel.analytics.bigdl.tensor.Tensor
import com.intel.analytics.bigdl.tensor.TensorNumericMath.TensorNumeric

import scala.reflect.ClassTag

class Rint[T: ClassTag]()(implicit ev: TensorNumeric[T])
  extends Operation[Tensor[Float], Tensor[Float], T]{

  output = Tensor[Float]()

  override def updateOutput(input: Tensor[Float]): Tensor[Float] = {
    output.resizeAs(input)
    output.map(input, (a, b) => {Math.rint(b).toFloat})
    output
  }
}

object Rint {
  def apply[T: ClassTag]()(implicit ev: TensorNumeric[T]): Rint[T] = new Rint[T]()
}
